"""
Build a single PRNG for testing/development

Usage:

    python setup-single-rng.py build_ext --in-place -f
"""
import glob
import os
import sys
from distutils.core import setup
from distutils.extension import Extension
from os import getcwd, name
from os.path import join

import Cython.Tempita as tempita
import numpy
from Cython.Build import cythonize

rngs = ['RNG_PCG32', 'RNG_PCG64',
        'RNG_MT19937', 'RNG_XORSHIFT128',
        'RNG_XORSHIFT1024', 'RNG_MRG32K3A', 'RNG_MLFG_1279_861']

files = glob.glob('./*.in')
for templated_file in files:
    output_file_name = os.path.splitext(templated_file)[0]
    if os.path.exists(output_file_name):
        if os.path.getmtime(templated_file) < os.path.getmtime(output_file_name):
            continue
    with open(templated_file, 'r') as source_file:
        template = tempita.Template(source_file.read())
    with open(output_file_name, 'w') as output_file:
        output_file.write(template.substitute())

with open('config.pxi', 'w') as config:
    config.write('# Autogenerated\n\n')
    config.write("DEF RS_RNG_MOD_NAME='mt19937'\n")

pwd = getcwd()

sources = [join(pwd, 'randomstate.pyx'),
           join(pwd, 'src', 'common', 'entropy.c'),
           join(pwd, 'distributions.c')]

sources += [join(pwd, 'src', 'random-kit', p) for p in ('random-kit.c','random-kit-jump.c')]
sources += [join(pwd, 'interface', 'random-kit', 'random-kit-shim.c')]

defs = [('RS_RANDOMKIT', '1')]

include_dirs = [pwd] + [numpy.get_include()]
if os.name == 'nt' and sys.version_info < (3, 5):
    include_dirs += [join(pwd, 'src', 'common')]
include_dirs += [join(pwd, 'src', 'random-kit')]

extra_link_args = ['Advapi32.lib'] if name == 'nt' else []
extra_compile_args = [] if name == 'nt' else ['-std=c99']

setup(ext_modules=cythonize([Extension("randomstate.randomstate",
                                       sources=sources,
                                       include_dirs=include_dirs,
                                       define_macros=defs,
                                       extra_compile_args=extra_compile_args,
                                       extra_link_args=extra_link_args)
                             ])
      )
